//=============================================================================
// File: diskpartition.h
//    
// Created Thu Apr 13 11:49:56 2000
//    
// Copyright (c) 2000 
// Justin E. Forrester (jforrest@cs.wisc.edu)
// Jonathan Ledlie (ledlie@cs.wisc.edu)
// All Rights Reserved
// 
// Description:
//    disk partition class
//     
//=============================================================================
#ifndef DISK_PARTITION_H_764
#define DISK_PARTITION_H_764

#include "tuple.h"
#include "error.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <iostream>
#include <unistd.h>

//================================================
// structure for 2nd stage activation records
//================================================
typedef struct TimestampListNode
{
    Timestamp lastDTS;
    Timestamp probeTS;
    TimestampListNode *next;
} TimestampListNode;

//================================================
class DiskPartition
{
    friend ostream& operator << (ostream& os, DiskPartition& diskPartition);
public:
    DiskPartition ();
    ~DiskPartition ();
    Status setFilename(char *filename);
    
    // appending funcs
    Status beginAppend(void);
    Status appendTuple(StampedTuple *st);
    Status endAppend(void);

    // reading funcs
    Status beginRead(void);
    Status nextTuple(StampedTuple *st);
    Status endRead(bool addListNode);

    // check dups in 2nd & 3rd stages
    Status checkDups(StampedTuple *tupleFromThisDP, StampedTuple *matchingTuple);
  
    int getTupleCount(void);
    Status cleanup ();
    
private:
    // fd of the corresponding file
    int fd;

    // number of tuples on the disk
    int tupleCount;

    // filename of the file containing the tuples
    char fileName [MAX_FILENAME];

    // list of 2nd stage activation records
    TimestampListNode *tsList;
    
    // current last DTS
    Timestamp lastDTS;

};

#endif
