//=============================================================================
// File: error.h
//    
// Created Tue Apr 11 15:16:10 2000
//    
// Copyright (c) 2000 
// Justin E. Forrester (jforrest@cs.wisc.edu)
// Jonathan Ledlie (ledlie@cs.wisc.edu)
// All Rights Reserved
// 
// Description:
//     error handling stuff
//     
//=============================================================================
#ifndef ERROR_H_764
#define ERROR_H_764

#ifdef DEBUG
#define ASSERT(c)  {                                                         \
                     if (!(c)) {                                             \
                        fprintf(stderr,                                      \
                                "ASSERTION FAILURE: %s:%d\n",                \
                                __FILE__,                                    \
                                __LINE__);                                   \
                        fprintf(stderr, "\tfailed condition: %s\n", #c);     \
                        exit(1);                                             \
                     }                                                       \
                   }
#else
#define ASSERT(c)
#endif

enum Status 
{
    OK = 0,
    
    // connection problems
    SOCKET_ERR,              // error reading from/creating socket 
    CONNECTIONS_BLOCKED,     // both connections currently blocked
    SOURCE_CONNECTION_DONE,  // given tuple source is done sending tuples

    // mem partition 
    NOMATCH,

    // disk parition status
    NO_MORE_TUPLES,          // no more tuples to read from disk partition
    FILE_ERROR,
    ERR_DUPLICATE,           // this is a duplicate match

    // misc error
    SYSTEM_ERR               // misc. system errors

};
    
#endif
