//=============================================================================
// File: main.cpp
//    
// Created Thu Apr 13 14:50:19 2000
//    
// Copyright (c) 2000 
// Justin E. Forrester (jforrest@cs.wisc.edu)
// Jonathan Ledlie (ledlie@cs.wisc.edu)
// All Rights Reserved
// 
// Description:
//    main tester program 
//     
//=============================================================================
#include <stdlib.h>
#include <pthread.h>
#include "xjoin.h"
#include "output.h"
#include "tuple.h"

int main(int argc, char *argv[])
{
    // deal with the arguments
    if (argc < 2 || argc > 3)
    {
        fprintf(stderr, "Usage: %s <total bytes of memory> [-no2nd]\n", argv[0]);
        exit(1);
    }

    // get memory use 
    int mem = atoi(argv[1]) * sizeof(char);

    // start up our xjoin obj
    printf("Memory allowance: %d\n", mem);
    XJoin x(mem);

    // if the user specified not using the 2nd stage, turn it off
    if (argc == 3 && 0 == strcmp(argv[2], "-no2nd"))
    {
        printf("Use second stage: No\n");
        x.toggleSecondStageUse(false);
    }
    else
    {
        printf("Use second stage: Yes\n");
    }

    // do our xjoin
    x.init();

    // start up our stat printer thread
    // NOTE: this is technically multithreaded, but no hardcore thread sync 
    // stuff is needed.  This thread will only be reading 1 int value, not
    // updating any data structures
    pthread_t tid;
    if (0 != pthread_create(&tid, NULL, Output::statPrinterThreadFunc, NULL))
    {
        fprintf(stderr, "Error starting monitoring thread, exiting\n");
        exit(1);
    }

    x.exec();

    Output::printStats();
}

