//=============================================================================
// File: mempartition.h
//    
// Created Tue Apr 11 11:58:10 2000
//    
// Copyright (c) 2000 
// Justin E. Forrester (jforrest@cs.wisc.edu)
// Jonathan Ledlie (ledlie@cs.wisc.edu)
// All Rights Reserved
// 
// Description:
//    memory partition for xjoin 
//     
//=============================================================================
#ifndef MEMPARTITION_H_764
#define MEMPARTITION_H_764

#include "tuple.h"
#include "error.h"
#include "diskpartition.h"

// number of chains in the hash table
// keeping it simple and non-adaptive
// would be trivial to modify and make adaptive to
// number of input tuples
#define NUM_CHAINS 1009

class MemPartition
{
friend ostream& operator << (ostream& os, MemPartition& memPartition);
public:
    MemPartition ();
    ~MemPartition ();

    // corresponding disk partition funcs
    Status setDiskPartition(DiskPartition *dp);
    DiskPartition* getDiskPartition();

    // inserting tuples
    Status insert(StampedTuple *pTuple);

    // probe the hash table for a match
    Status probe(StampedTuple *pTuple, StampedTuple* &pMatchingTuple);

    // flush the memory to disk
    Status flush(int &bytesFlushed);

    // probe another mem partition with this mem part
    Status finalMemProbe(MemPartition *probeeMemPart);
    
    // accessors
    int getSize(void);
    int getTupleCount(void);

private:
    // id of this mem partition
    int id;

    // private hash function 
    int hash (int);

    // hash bucket chains
    StampedTuple *tuples[NUM_CHAINS];
    
    // total number of tuples in chains
    int tupleCount;

    // corresponding disk partition
    DiskPartition *diskPartition;
};

#endif
