//=============================================================================
// File: output.cpp
//    
// Created Tue Apr 11 13:28:28 2000
//    
// Copyright (c) 2000 
// Justin E. Forrester (jforrest@cs.wisc.edu)
// Jonathan Ledlie (ledlie@cs.wisc.edu)
// All Rights Reserved
// 
// Description:
//    implementation of output class 
//     
//=============================================================================
#include "output.h"
#include <iostream.h>
#include <unistd.h>

//=============================================================================
// Static class members
//=============================================================================
OutputTuple **Output::tuples = new OutputTuple *[INITIAL_OUTPUT_SIZE];
int Output::curMax = INITIAL_OUTPUT_SIZE;
int Output::numTuples = 0;
int Output::numSecondStageTuples = 0;
int Output::numSecondStageRuns = 0;

//=============================================================================
// Method: writeTuple
//     Description:
//         this function is called when the join produces an output tuple
//     Parameters:
//         pTuple - output tuple just joined
//     Return Value:
//         error status
//=============================================================================
Status Output::writeTuple(OutputTuple *pTuple, bool isSecondStage)
{
    ASSERT(pTuple->tuple1.tuple != NULL);
    ASSERT(pTuple->tuple2.tuple != NULL);

    // see if we are at our capacity
    if (numTuples == curMax)
    {
        // make it bigger
        OutputTuple **newTuples = new OutputTuple *[curMax * 2];
        for (int i = 0; i < curMax; i++)
        {
            newTuples[i] = tuples[i];
        }

	delete [] tuples;
        tuples = newTuples;
        curMax *= 2;
    }
    
    // insert item
    tuples[numTuples] = pTuple;

    numTuples++;

    if (isSecondStage)
        numSecondStageTuples++;

    return OK;
} // end writeTuple


//=============================================================================
// Method: recordSecondStageRun
//     Description:
//         increment 2nd stage counter
//     Parameters:
//         none
//     Return Value:
//         none
//=============================================================================
void Output::recordSecondStageRun(void)
{
    numSecondStageRuns++;
} // end recordSecondStageRun



//=============================================================================
// Method: printStats
//     Description:
//         print the number of output tuples, etc
//     Parameters:
//         none
//     Return Value:
//         none
//=============================================================================
Status Output::printStats(void)
{
    printf("numTuples %d secondStageMatches %d numSecondStageRuns %d\n", numTuples, numSecondStageTuples, numSecondStageRuns);
    fflush(stdout);

    return OK;

} // end printStats


//=============================================================================
// Method: statPrinterThreadFunc
//     Description:
//         function that the stat printer thread will call (never returns)
//     Parameters:
//         none
//     Return Value:
//         none
//=============================================================================
void *Output::statPrinterThreadFunc(void *arg)
{
    while (1)
    {
        printStats();
        sleep(1);
    }
    return NULL;
} // end statPrinterLoop
