#!/s/std/bin/perl
use strict;

#=================================================
# GLOBALS
#=================================================
my @saMachinesUsed;

my $sHome = $ENV{HOME};

my $sOutputBase = "$sHome/private/cs764";

my $sBinDir = "$sHome/private/cs764/cs764-project/src";
my $sTestOutputDir1 = "$sOutputBase/test_results1";
my $sTestOutputDir2 = "$sOutputBase/test_results2";
my $sOutputParser = "$sBinDir/parseOutput.pl";

#my $nTotalNumTuples = 1;
my $nTotalNumTuples = 100000;

my $n1MB = 2**20;
my $n3MB = 3*(2**20);
my $n4MB = 4*(2**20);
my $n8MB = 8*(2**20);
my $n16MB = 16*(2**20);
my $n32MB = 32*(2**20);
my $n64MB = 64*(2**20);


goto SECONDSET;

print("Doing first set of tests\n");
#==============================================================================
# run memsize & 2nd stage toggling tests
#==============================================================================

#my @naMemSizes = ( $n1MB, $n4MB, $n8MB, $n16MB, $n32MB, $n64MB );
my @naMemSizes = ( $n1MB, $n4MB, $n16MB, $n64MB );
my @saSecondStageFlags = ( "", "-no2nd");

my $nFixedBurstLen = 1000;
my $nFixedBurstDelay = 10;

my $sSecondStageFlag;
foreach $sSecondStageFlag (@saSecondStageFlags)
{
    my $nMemSize;
    foreach $nMemSize (@naMemSizes)
    {
	my $novaName = getBestNova();
	
	my $rshPID;
	if ($rshPID = fork)
	{
	    # give the job a chance to get going
	    sleep(10);
	}
	else
	{
	    # child responsible for doing the tests
	    # run tests
	    my $sRawOutput;
	    my $sExcelOutput;
	    if ($sSecondStageFlag eq "")
	    {
		$sRawOutput = "$sTestOutputDir1/xjoin_$nMemSize" . "_with2nd.raw";
		$sExcelOutput = "$sTestOutputDir1/xjoin_$nMemSize" . "_with2nd.txt";
	    }
	    else
	    {
		$sRawOutput = "$sTestOutputDir1/xjoin_$nMemSize" . "_no2nd.raw";
		$sExcelOutput = "$sTestOutputDir1/xjoin_$nMemSize" . "_no2nd.txt";
	    }
	    
	    # what host to run on
	    print("Sending job to $novaName\n");

	    my $pid;
	    if ($pid = fork)
	    {
		sleep(5);
		doSystem("rsh $novaName $sBinDir/tuple_source $nTotalNumTuples $nFixedBurstLen $nFixedBurstDelay > /dev/null &");
		doSystem("rsh $novaName $sBinDir/tuple_source $nTotalNumTuples $nFixedBurstLen $nFixedBurstDelay > /dev/null &");
		waitpid($pid, 0);
	    }
	    elsif (defined $pid)
	    {
		exec("rsh $novaName $sBinDir/xjoin $nMemSize $sSecondStageFlag > $sRawOutput");
	    }
	    else
	    {
		die "Something is VERY wrong...";
	    }
	    
	    # convert the file
	    doSystem("$sBinDir/parseOutput.pl $sRawOutput $sExcelOutput");
	    print("$novaName done:\n");
	    print("   $sRawOutput\n");
	    exit(1);
	} # else child process
    }
}
exit(1);

SECONDSET:

    print("Doing second set of tests\n");
#==============================================================================
# Do varying tuple_source tests
#==============================================================================
my @naBurstSizes = ( 1000, 10000 );
my @naDelayLengths = ( 5, 15 );

my $nFixedMemSize = $n4MB;

my $nBurstSize;
foreach $nBurstSize (@naBurstSizes)
{
    my $nDelayLength;
    foreach $nDelayLength (@naDelayLengths)
    {
	my $novaName = getBestNova();	    

	my $rshPID;
	if ($rshPID = fork)
	{
	    # give the job a chance to get going
	    sleep(10);
	}
	else
	{
	    # run the test with second stage
	    my $sRawOutput;
	    my $sExcelOutput;
	    $sRawOutput = "$sTestOutputDir2/xjoin_$nBurstSize" . "_$nDelayLength.raw";
	    $sExcelOutput = "$sTestOutputDir2/xjoin_$nBurstSize" . "_$nDelayLength.txt";
	    # what host to run on
	    print("Sending job to $novaName\n");
	    
	    my $pid;
	    if ($pid = fork)
	    {
		sleep(5);
		doSystem("rsh $novaName $sBinDir/tuple_source $nTotalNumTuples $nBurstSize $nDelayLength > /dev/null &");
		doSystem("rsh $novaName $sBinDir/tuple_source $nTotalNumTuples $nBurstSize $nDelayLength > /dev/null &");
		waitpid($pid, 0);
	    }
	    elsif (defined $pid)
	    {
		exec("rsh $novaName $sBinDir/xjoin $nFixedMemSize > $sRawOutput");
	    }
	    else
	    {
		die "Something is VERY wrong...";
	    }
	    
	    # convert the file
	    doSystem("$sBinDir/parseOutput.pl $sRawOutput $sExcelOutput");
	    print("$novaName done.\n");
	    print("   $sRawOutput\n");
	    exit(1);
	} # else child, to processing
    }
}



sub doSystem
{
    my ($sCommand) = @_;

    my $nErrStatus = system("$sCommand 2> /dev/stdout");
    $nErrStatus = $nErrStatus >> 8;
    return $nErrStatus;
}


sub getBestNova
{
    while (1)
    {
	my ($novaName, $aliases, $addrtype, $length, @addrs) 
	    = gethostbyname("best-nova");
	if (! haveUsedSystem($novaName))
	{
	    push(@saMachinesUsed, $novaName);
	    return $novaName;
	}
    }
}

sub haveUsedSystem
{
    my ($sName) = @_;
    
    my $sCurr;
    foreach $sCurr (@saMachinesUsed)
    {
	if ($sCurr eq $sName)
	{
	    return 1;
	}
    }
    return 0;
}
    

	
