//=============================================================================
// File: timestamp.cpp
//    
// Created Mon Apr  3 13:54:03 2000
//    
// Copyright (c) 2000 
// Justin E. Forrester, (jforrest@cs.wisc.edu)
// Jonathan Ledlie (ledlie@cs.wisc.edu)
// All Rights Reserved
// 
// Description:
//    impl of timestamp class functions 
//     
//=============================================================================
#include "timestamp.h"
#include <stdio.h>


//=============================================================================
// Method: ctor
//     Description:
//         Constructor
//     Parameters:
//         none
//     Return Value:
//         -
//=============================================================================
Timestamp::Timestamp(void)
{
    if (-1 == gettimeofday(&tv, NULL))
    {
        perror("Error getting time of day: ");
    }
} // end Timestamp ctor


//=============================================================================
// Method: copy ctor
//     Description:
//         Copy constructor
//     Parameters:
//         t - ref to another Timestamp obj
//     Return Value:
//         -
//=============================================================================
Timestamp::Timestamp(Timestamp &t)
{
    this->tv = t.tv;
} // end copy ctor


//=============================================================================
// Method: print
//     Description:
//         dumps the contents of the timestamp
//     Parameters:
//         none
//     Return Value:
//         none
//=============================================================================
void Timestamp::print(void)
{
    fprintf(stdout, "%d.%d\n", (int) tv.tv_sec, (int) tv.tv_usec);
} // print

    
//=============================================================================
// Method: setToCurrTime
//     Description:
//         update the time of the timestamp to the current time
//     Parameters:
//         none
//     Return Value:
//         none
//=============================================================================
void Timestamp::setToCurrTime(void)
{
    if (-1 == gettimeofday(&tv, NULL))
    {
        perror("Error getting time of day: ");
    }
} // end setToCurrTime


//=============================================================================
// Method: setToMaxTime
//     Description:
//         set to time in future (i.e. hasn't happened yet)
//     Parameters:
//         none
//     Return Value:
//         none
//=============================================================================
void Timestamp::setToMaxTime(void)
{
    this->tv.tv_sec = INT_MAX;
    this->tv.tv_usec = INT_MAX;
} // end setToMaxTime


//=============================================================================
// comparison ops
//=============================================================================
bool Timestamp::operator==(const Timestamp& t) const
{
  //    fprintf(stderr, "eq\n");
    return (this->tv.tv_sec == t.tv.tv_sec &&
            this->tv.tv_usec == t.tv.tv_usec);
}

bool Timestamp::operator!=(const Timestamp& t) const
{
  //    fprintf(stderr, "neq\n");
    return (this->tv.tv_sec != t.tv.tv_sec ||
            this->tv.tv_usec != t.tv.tv_usec);
}

bool Timestamp::operator<(const Timestamp& t) const
{
  //    fprintf(stderr, "gt\n");
    if (this->tv.tv_sec < t.tv.tv_sec) return 1;
    if (this->tv.tv_sec == t.tv.tv_sec &&
        this->tv.tv_usec < t.tv.tv_usec) return 1;
    return 0;
}

bool Timestamp::operator>(const Timestamp& t) const
{
  //    fprintf(stderr, "lt\n");
    if (this->tv.tv_sec > t.tv.tv_sec) return 1;
    if (this->tv.tv_sec == t.tv.tv_sec &&
        this->tv.tv_usec > t.tv.tv_usec) return 1;
    return 0;
}

bool Timestamp::operator>=(const Timestamp& t) const
{
  //    fprintf(stderr, "ge\n");
    if (this->tv.tv_sec > t.tv.tv_sec) return 1;
    if (this->tv.tv_sec == t.tv.tv_sec &&
        this->tv.tv_usec >= t.tv.tv_usec) return 1;
    return 0;
}

bool Timestamp::operator<=(const Timestamp& t) const
{
  //    fprintf(stderr, "le\n");
    if (this->tv.tv_sec < t.tv.tv_sec) return 1;
    if (this->tv.tv_sec == t.tv.tv_sec &&
        this->tv.tv_usec <= t.tv.tv_usec) return 1;
    return 0;
}

//=============================================================================
// end comparison ops
//=============================================================================
