//=============================================================================
// File: tuple_source.cpp
//    
// Created Mon Apr  3 14:40:48 2000
//    
// Copyright (c) 2000 
// Justin E. Forrester, (jforrest@cs.wisc.edu)
// Jonathan Ledlie (ledlie@cs.wisc.edu)
// All Rights Reserved
// 
// Description:
//    process for the production of WI benchmark tuples and sending to 
//    the receiver 
//     
//=============================================================================
#include "tuple.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ddi.h>
#include <stdio.h>
#include <errno.h>
#include <strings.h>
#include <netdb.h>

#define NUMRANDOMIZEPASSES 10

//=============================================================================
// Method: genRandomIntArray
//     Description:
//         generate array of random ints
//     Parameters:
//         nums - array to receive random nums
 //         len - length of array
//         maxEntry - maximum number (cardinality factor)
//     Return Value:
//         
//=============================================================================
void genRandomIntArray(int nums[], int len, int maxEntry)
{
    for (int i = 0; i < len; i++)
    {
        nums[i] = i;
    }
    
    for (int pass = 0; pass < NUMRANDOMIZEPASSES; pass++)
    {
        for (int i = 0; i < len; i++)
        {
            // swap curr entry to new pos
            int newPos = rand() % len;
            int tmpVal = nums[newPos];
            nums[newPos] = nums[i];
            nums[i] = tmpVal; 
        }
    }

} // end genRandomIntArray


//=============================================================================
// Method: main
//     Description:
//         duh
//     Parameters:
//         duh
//     Return Value:
//         error status
//=============================================================================
int main(int argc, char *argv[])
{
    // get command line args
    if (argc != 4)
    {
        fprintf(stderr, 
                "Usage: %s <total num tuples> <num tuples per burst> <pause between bursts in seconds>\n",
                argv[0]);
        exit(1);
    }
    int tupleCount = atoi(argv[1]);
    int burstSize = atoi(argv[2]);
    int sleepTime = atoi(argv[3]);

    // seed the random number generator
    timeval tv;
    if (-1 == gettimeofday(&tv, NULL))
    {
        perror("Error getting time of day: ");
        exit(1);
    }
    srand(tv.tv_usec);

    int nums[tupleCount];
    genRandomIntArray(nums, tupleCount, tupleCount);

    WITuple tuple;
    int sockfd;
    struct sockaddr_in server_addr;

    // open a TCP socket
    if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    {
        perror("server: unable to open socket");
        return 1;
    }

    // fill in server address
    memset(&server_addr, 0, sizeof(struct sockaddr_in));
    server_addr.sin_family = AF_INET;
    server_addr.sin_port = htons(TUPLEPORT);
    server_addr.sin_addr.s_addr = inet_addr("127.0.0.1");

    // connect to the server
    if (connect(sockfd, (struct sockaddr *) &server_addr, sizeof(struct sockaddr_in)) < 0)
    {
        perror("client: failure in connect");
        return 1;
    }

    // write the tuples to the port
    for (int i = 0; i < tupleCount; i++)
    {
        // assign the unique1
        tuple.unique1 = nums[i];

        //printf("writing tuple %d: %d\n", i, tuple.unique1);
        if (write(sockfd, &tuple, sizeof(tuple)) == -1)
        {
            perror("Error writing tuple\n");
        }
        if ((i+1) % burstSize == 0 && (i+1) != tupleCount)
        {
            printf("sent %d tuples\n", i+1);
            sleep(sleepTime);
        }
    }
        
    close(sockfd);

    printf("Tuple source done.\n");
    return 0;

} // end main

    
