//=============================================================================
// File: xjoin.h
//    
// Created Mon Apr  3 13:45:45 2000
//    
// Copyright (c) 2000 
// Justin E. Forrester (jforrest@cs.wisc.edu)
// Jonathan Ledlie (ledlie@cs.wisc.edu)
// All Rights Reserved
// 
// Description:
//    include for xjoin specific structes 
//     
//=============================================================================
#ifndef XJOIN_H_764
#define XJOIN_H_764

#include "tuple.h"
#include "timestamp.h"
#include "mempartition.h"
#include "error.h"

// number of global partitions for the tuples
// mid-sized prime, choosing to go the static route here
#define NUM_PARTITIONS 47

class XJoin
{
public:
    XJoin(int max);
    ~XJoin ();

    // turn on/off second stage -- use for experiments
    void toggleSecondStageUse(bool useSecondStage);

    // get XJoin going, accept connections from sources
    Status init(void);
    
    // execute the XJoin algorithm
    Status exec(void);

    // various stages of the algorithm, could make private
    Status firstStage(void);
    Status secondStage(void);
    Status thirdStage(void);

private:
    // get the tuple sources connected
    Status acceptConnections(void);

    // hash on the tuple to get partition
    int getPartition(WITuple *tuple);

    // get a tuple from the input, insert in hash table and probe other
    Status getTuple(int connfd, 
                    MemPartition *insertPartitions, 
                    MemPartition *probePartitions);

    // probe memory partition with tuples from disk partition
    Status probeFromDisk(MemPartition *probeeMemPart,
                         DiskPartition *proberDiskPart,
                         bool isSecondStage);

    // connection management stuff
    int connfd1;
    int connfd2;
    int conn1done;
    int conn2done;

    // memory handling stuff
    int maxMem;
    int currMem;

    // performance analysis
    int tuplesFlushed;
    int proberTupleCount;
    int inMemoryMatchCount;

    // flag to use 2nd stage or not
    bool useSecondStage;

    // actual partitions for the 2 relations
    MemPartition memPartitions1[NUM_PARTITIONS];
    MemPartition memPartitions2[NUM_PARTITIONS];
    DiskPartition diskPartitions1[NUM_PARTITIONS];
    DiskPartition diskPartitions2[NUM_PARTITIONS];
    
};


#endif
